/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "AppController.h"
#include <conprint.h>
#include "std.h"

AppController* AppController::mInstance = NULL;

AppController::AppController()
{
  MAExtent screenSize = maGetScrSize();
  mScrWidth = EXTENT_X(screenSize);
  mScrHeight = EXTENT_Y(screenSize);

  mImageCache = NULL;
  mRSSDownloadQueue = NULL;
}

AppController::~AppController()
{
	//Delete all the screens managed by the controller
	Vector_each(Screen*, itr, mScreens)
		delete *itr;

	//Delete all the fonts managed by the controller
	for(HashMap<String, Font*>::Iterator itr = mFonts.begin();
		itr != mFonts.end();
		itr++)
		delete itr->second;

	//Delete all the skins managed by the controller
	for(HashMap<String, WidgetSkin*>::Iterator itr = mSkins.begin();
			itr != mSkins.end();
			itr++)
		delete itr->second;

	if(mImageCache != NULL)
		delete mImageCache;

	if(mRSSDownloadQueue != NULL)
		delete mRSSDownloadQueue;
}

//Instance manager
AppController* AppController::getInstance()
{
	if(mInstance == NULL)
		mInstance = new AppController();

	return mInstance;
}

//Properties
void AppController::setMoblet(Moblet* moblet)
{
	mMoblet = moblet;
}

Moblet* AppController::getMoblet()
{
	return mMoblet;
}

//Registration
int AppController::registerScreen(Screen* screen)
{
  mScreens.add(screen);

  return (int)mScreens.size() -1;
}

void AppController::registerScreen(String screenName, Screen* screen)
{
	mIndex.insert(screenName, registerScreen(screen));
}

//Show methods
void AppController::show(int screenIndex)
{
  Screen* oldScreen = NULL;
  Screen* newScreen = NULL;

  if(mScreens[screenIndex] != NULL)
  {
    if(mHistory.size() > 0)
      oldScreen = getScreen(mHistory.peek());

    mHistory.push(screenIndex);
    newScreen = getScreen(screenIndex);

    if(oldScreen != NULL)
       ScreenTransition::makeTransition(oldScreen, newScreen, 1, 400, MOVEOVERTOP);
    else
       newScreen->show();
  }
}

void AppController::show(String screenName)
{
	if(mIndex.find(screenName) != mIndex.end())
		show(mIndex.find(screenName)->second);
}

Screen* AppController::getScreen(int screenIndex)
{
  if(screenIndex >= 0)
	return mScreens[screenIndex];
}

Screen* AppController::getScreen(String screenName)
{
	Screen* tempScreen = NULL;
	if(mIndex.find(screenName) != mIndex.end())
		tempScreen = getScreen(mIndex.find(screenName)->second);

	return tempScreen;
}

//Move back one
void AppController::back()
{
	if(mHistory.size() > 1)
	{
		int currentScreen = mHistory.peek();
		mHistory.pop();
        Screen* oldScreen = NULL;
        Screen* newScreen = NULL;

        if(mScreens[currentScreen] != NULL)
        {
          newScreen = getScreen(mHistory.peek());
          oldScreen = getScreen(currentScreen);

          if(oldScreen != NULL)
              ScreenTransition::makeTransition(oldScreen, newScreen, -1, 400, REVEALFROMBELOW);
          else
              newScreen->show();
        }
	}
}

//Close the application
void AppController::close()
{
	if(mMoblet != NULL)
		mMoblet->close();
}

int AppController::getIndex(String& screenName)
{
	return mIndex[screenName];
}

void AppController::addFont(String name, Font* font)
{
  mFonts.insert(name, font);
}

Font* AppController::getFont(String name)
{
  return mFonts.find(name)->second;
}

void AppController::addSkin(String name, WidgetSkin* skin)
{
  mSkins.insert(name, skin);
}

WidgetSkin* AppController::getSkin(String name)
{
  return mSkins.find(name)->second;
}

int AppController::getScreenWidth()
{
  return mScrWidth;
}

int AppController::getScreenHeight()
{
  return mScrHeight;
}

ImageCache* AppController::getImageCache()
{
  if(mImageCache == NULL)
	  mImageCache = new ImageCache();

  return mImageCache;
}

RSSDownloadQueue* AppController::getRSSDownloadQueue()
{
	if(mRSSDownloadQueue == NULL)
		mRSSDownloadQueue = new RSSDownloadQueue();

	return mRSSDownloadQueue;
}
