/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "MenuScreen.h"
#include "../std.h"
#include "../Widgets/Colours.h"
#include "../Widgets/UIBuilder.h"
#include "../Utilities/Convert.h"
#include "RSSIndex.h"

MenuScreen::MenuScreen()
{
	//Create the UI
	mLayout = new Layout(0, 0, APPCONTROLLER->getScreenWidth(), APPCONTROLLER->getScreenHeight(), NULL, 1, 2);
	mLayout->setBackgroundColor(BLACK);//Colours are defined in /Widgets/Colours.h
	mLayout->setDrawBackground(true);
	//mLayout->setMarginX(15);

	//Create the title.  createLabel is a factory in UIBuilder
	Label* title = createLabel("World News Reader", 30);
	title->setHorizontalAlignment(Label::HA_CENTER);
	title->setVerticalAlignment(Label::VA_CENTER);
	title->setFont(APPCONTROLLER->getFont("default"));
	title->setDrawBackground(true);
	title->setAutoSizeY(true);
	title->setPaddingLeft(0);
	title->setPaddingRight(0);
	title->setSkin(APPCONTROLLER->getSkin("title"));
	title->update(); //Calculate the correct size
	mLayout->add(title);

	//Create the gridmenu
	mGridMenu = new GridMenu(0, 0, APPCONTROLLER->getScreenWidth(), APPCONTROLLER->getScreenHeight() - title->getHeight(), mLayout, 3, 3);
	mGridMenu->setDrawBackground(false);
	mGridMenu->setMarginX(3);
	mGridMenu->setPaddingLeft(5);

	//Listen for events from the gridmenu
	mGridMenu->addListener(this);

	//Ask the grid menu to respond to keypresses
	mGridMenu->bindKeyDown(MAK_DOWN);
	mGridMenu->bindKeyDown(MAK_8);
	mGridMenu->bindKeyLeft(MAK_LEFT);
	mGridMenu->bindKeyLeft(MAK_4);
	mGridMenu->bindKeyRight(MAK_RIGHT);
	mGridMenu->bindKeyRight(MAK_6);
	mGridMenu->bindKeyUp(MAK_UP);
	mGridMenu->bindKeyUp(MAK_2);
	mGridMenu->bindKeySelect(MAK_FIRE);
	mGridMenu->bindKeySelect(MAK_5);

	//Read the config into a char buffer.  CONFIG is a reference to a resource in
	//res.lst
	char buffer[maGetDataSize(CONFIG) + 1];
	maReadData(CONFIG, &buffer, 0, maGetDataSize(CONFIG));
	//ensure that it is null terminated
	buffer[maGetDataSize(CONFIG)] = '\0';

	//Parse the config file, and add the RSS feeds accordingly.
	mParser = new JSONParser();
	mParser->addListener(this);
	mObjectCounter = 0;
	String jsonString;
	jsonString.append(buffer, strlen(buffer));
	mParser->parseString(jsonString);

	this->setMain(mLayout);
}

MenuScreen::~MenuScreen()
{}

//GridMenu
void MenuScreen::GridMenuItemSelected(int item, Widget* selectedItem)
{
	//Show the correct instance of RSSIndex

	//See if we've already got a screen for this feed
	if(mScreens.find(item) == mScreens.end())
	{
		//We've not created a screen for this feed yet
		//Create a new screen.  The AppController will return an index number
		//for us to use
		int newScreen = APPCONTROLLER->registerScreen(new RSSIndex());

		//Get the screen back from the AppController
		RSSIndex* newIndex = (RSSIndex*)APPCONTROLLER->getScreen(newScreen);

		//Set the url
		newIndex->getFeed(mURLs[item]);

		//Add the new screen to the index
		mScreens.insert(item, newScreen);
	}

	//Show the index screen
	APPCONTROLLER->show(mScreens[item]);
}

//JSONParser
void MenuScreen::header(String& header)
{
	//LOG("Header: %s", header.c_str());
	if(header == "name")
		mConfigState = NAME;
	else if(header == "url")
		mConfigState = URL;
	else if(header == "logo")
		mConfigState = LOGO;
}

void MenuScreen::data(String& value)
{
	switch(mConfigState)
	{
		case NAME:
			//Don't show a label.  If you want to show a label underneath the image, uncomment the next line
			mButton->setLabel(value.c_str());
			break;
		case LOGO:
			mButton->setMenuImage(LOGO_START + Convert::toInt(value));
			break;
		case URL:
			mURLs.add(value);
			break;
	}
}

void MenuScreen::arrayStart()
{}

void MenuScreen::arrayEnd()
{}

void MenuScreen::objectStart()
{
	mObjectCounter++;
	if(mObjectCounter == 2)
		mButton = (MenuButton*)createMenuImage(LOGO_START, "", APPCONTROLLER->getFont("default"), NULL);
}

void MenuScreen::objectEnd()
{
	mObjectCounter--;
	if(mObjectCounter == 1)
		mGridMenu->add(mButton);

	if(mObjectCounter == 0)
		mGridMenu->setSelectedIndex(0);
}
