/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "RSSIndex.h"
#include "../std.h"
#include "../Widgets/Colours.h"
#include "RSSScreen.h"
#include "MAHeaders.h"


RSSIndex::RSSIndex()
{
	mRSSScreenID = -1;

	mLayout = new Layout(0, 0, APPCONTROLLER->getScreenWidth(), APPCONTROLLER->getScreenHeight(), NULL, 1, 3);
	mTitle = new Label(0, 0, APPCONTROLLER->getScreenWidth(), 30, mLayout);
	mTitle->setHorizontalAlignment(Label::HA_CENTER);
	mTitle->setVerticalAlignment(Label::VA_CENTER);
	mTitle->setFont(APPCONTROLLER->getFont("default"));
	mTitle->setDrawBackground(true);
	mTitle->setAutoSizeY(true);
	mTitle->setPaddingLeft(0);
	mTitle->setPaddingRight(0);
	mTitle->setSkin(APPCONTROLLER->getSkin("title"));
	mTitle->update(); //Calculate the correct size

	mSoftkey = new Label(0, 0, APPCONTROLLER->getScreenWidth(), 30, NULL);
	mSoftkey->setHorizontalAlignment(Label::HA_RIGHT);
	mSoftkey->setVerticalAlignment(Label::VA_CENTER);
	mSoftkey->setFont(APPCONTROLLER->getFont("default"));
	mSoftkey->setDrawBackground(true);
	mSoftkey->setAutoSizeY(true);
	mSoftkey->setPaddingLeft(0);
	mSoftkey->setPaddingRight(10);
	mSoftkey->setSkin(APPCONTROLLER->getSkin("title"));
	mSoftkey->setCaption("back");
	mSoftkey->update(); //Calculate the correct size

	mItems = new KineticListBox(0, 0, APPCONTROLLER->getScreenWidth(),
						 APPCONTROLLER->getScreenHeight() - mTitle->getHeight() - mSoftkey->getHeight(),
						 mLayout, KineticListBox::LBO_VERTICAL, KineticListBox::LBA_LINEAR, true);

	mLayout->add(mSoftkey);

	this->setMain(mLayout);
}

RSSIndex::~RSSIndex()
{}

//Download and process the RSS feed
void RSSIndex::getFeed(String& url)
{
	//Requests a download of the RSS feed.  The URL comes from the config.json file
	//and you pass a pointer to this class so that RSS data is passed back to this
	//instance
	APPCONTROLLER->getRSSDownloadQueue()->addRequest(url, this);
}

void RSSIndex::keyPressEvent(int keyCode, int nativeCode)
{
	switch(keyCode)
	{
		case MAK_UP:
		case MAK_LEFT:
		case MAK_2:
		case MAK_4:
			mItems->selectPreviousItem();
			break;
		case MAK_DOWN:
		case MAK_RIGHT:
		case MAK_8:
		case MAK_6:
			mItems->selectNextItem();
			break;
		case MAK_SOFTRIGHT:
			APPCONTROLLER->back();
			break;
		case MAK_5:
		case MAK_FIRE:
   		  #ifdef SHOWDETAIL
			//Reuse the same screen for any RSS item
			RSSScreen* screen = (RSSScreen*)APPCONTROLLER->getScreen("RSSScreen");

			//Get the selected widget
			RSSItemWidget* widget = (RSSItemWidget*)mItems->getChildren()[mItems->getSelectedIndex()];

			//Pass the RSS Item data
			RSSItem* newitem = new RSSItem();
			newitem->description = widget->getText();
			newitem->imageurl = widget->getImageUrl();
			newitem->title = widget->getTitle();
			newitem->url = widget->getUrl();

			//Pass the data to the detail screens
			screen->setRSSItem(newitem);

			//We can delete the RSSItem now
			delete newitem;

			//Show the screen
			APPCONTROLLER->show("RSSScreen");
			break;
		  #else
			//Launch the url
			//Get the selected widget
			RSSItemWidget* widget = (RSSItemWidget*)mItems->getChildren()[mItems->getSelectedIndex()];

			maPlatformRequest(widget->getUrl().c_str());
		  #endif
	}
}

//Get touch events from the listbox
void RSSIndex::itemSelected(KineticListBox* sender, Widget* selectedWidget, Widget* unselectedWidget)
{
}

void RSSIndex::blocked(KineticListBox* sender, int direction)
{
	//We can safely ignore blocked errors
}

//This method is called when a new RSS channel has been received
void RSSIndex::rssChannelReceived(RSSChannel& channel)
{
	//Delete any existing items we may have
	Vector_each(Widget*, itr, mItems->getChildren())
		delete *itr;
	mItems->clear();

	//Set the tile and the copyright message, and then resize the space
	mTitle->setCaption(channel.title);
	mTitle->update();

	mItems->setHeight(APPCONTROLLER->getScreenHeight() - mTitle->getHeight() - mSoftkey->getHeight());
}

//This method is called when an RSS Item is received.  The RSSItem passed will go out of scope
//at some point after this method is called, so you have to copy the data if you want to refer to it
//later
void RSSIndex::rssItemReceived(RSSItem& item)
{
	RSSItemWidget* newItem = new RSSItemWidget(0, 0, APPCONTROLLER->getScreenWidth(), 30, mItems);
	newItem->setText(item.description);
	newItem->setTextFont(APPCONTROLLER->getFont("itemfont"));
	newItem->setTitle(item.title);
	newItem->setTitleFont(APPCONTROLLER->getFont("title"));
	newItem->setNextImage(NEXTIMAGE);
	LOG("Set url %s", item.url.c_str());
	newItem->setUrl(item.url);
	newItem->setImageUrl(item.imageurl);
	newItem->setSkin(APPCONTROLLER->getSkin("rssitem"));
	newItem->addWidgetListener(this);

	mItems->setSelectedIndex(0);
	mItems->getChildren()[0]->setSelected(true);
}

//Receive trigger events from RSSItemWidget
void RSSIndex::triggered(Widget* sender)
{
	//Act as if fire has been pressed
	keyPressEvent(MAK_FIRE, 0);
}


