/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "RSSScreen.h"
#include "../Widgets/Colours.h"
#include "../std.h"
#include "MAHeaders.h"

RSSScreen::RSSScreen()
{
	mLayout = new Layout(0, 0, APPCONTROLLER->getScreenWidth(), APPCONTROLLER->getScreenHeight(), NULL, 1, 3);
	mTitle = new Label(0, 0, APPCONTROLLER->getScreenWidth(), 30, mLayout);
	mTitle->setHorizontalAlignment(Label::HA_CENTER);
	mTitle->setVerticalAlignment(Label::VA_CENTER);
	mTitle->setFont(APPCONTROLLER->getFont("default"));
	mTitle->setDrawBackground(true);
	mTitle->setAutoSizeY(true);
	mTitle->setMultiLine(true);
	mTitle->setPaddingLeft(5);
	mTitle->setPaddingRight(5);
	mTitle->setSkin(APPCONTROLLER->getSkin("title"));
	mTitle->update(); //Calculate the correct size

	mSoftkey = new Label(0, 0, APPCONTROLLER->getScreenWidth(), 30, NULL);
	mSoftkey->setHorizontalAlignment(Label::HA_RIGHT);
	mSoftkey->setVerticalAlignment(Label::VA_CENTER);
	mSoftkey->setFont(APPCONTROLLER->getFont("default"));
	mSoftkey->setDrawBackground(true);
	mSoftkey->setAutoSizeY(true);
	mSoftkey->setPaddingLeft(0);
	mSoftkey->setPaddingRight(10);
	mSoftkey->setSkin(APPCONTROLLER->getSkin("title"));
	mSoftkey->setCaption("back");
	mSoftkey->update(); //Calculate the correct size
	mContent = new ListBox(0, 0, APPCONTROLLER->getScreenWidth(),
			 APPCONTROLLER->getScreenHeight() - mTitle->getHeight() - mSoftkey->getHeight(),
			 mLayout, ListBox::LBO_VERTICAL, ListBox::LBA_LINEAR, false);

	mContent->setBackgroundColor(WHITE);

	//Set up the widget which will contain the main content
	mImage = new Image(5, 5, 20, 20, mContent, true, true, BLANK);
	mImage->setPaddingLeft(5);
	mImage->setPaddingTop(5);
	mText = new Label(0, 0, APPCONTROLLER->getScreenWidth(), 100, mContent);
	mText->setMultiLine(true);
	mText->setAutoSizeY(true);
	mText->setDrawBackground(false);
	mText->setPaddingLeft(5);
	mText->setPaddingRight(5);
	mText->setFont(APPCONTROLLER->getFont("itemfont"));
	mLink = new Label(0, 0, APPCONTROLLER->getScreenWidth(), 100, mContent);
	mLink->setMultiLine(true);
	mLink->setAutoSizeY(true);
	mLink->setDrawBackground(false);
	mLink->setPaddingLeft(5);
	mLink->setPaddingRight(5);
	mLink->setPaddingTop(10);
	mLink->setFont(APPCONTROLLER->getFont("itemfont"));

	mLayout->add(mSoftkey);
	this->setMain(mLayout);
}

RSSScreen::~RSSScreen()
{
	delete mLayout;
}

void RSSScreen::imageReady(MAHandle h)
{
	mImage->setResource(h);
	mImage->update();
}

void RSSScreen::setRSSItem(RSSItem* rssItem)
{
	//LOG("ImageURL %s", rssItem->imageurl.c_str());
	//Create a new image cache request for this item
	if(rssItem->imageurl.length() > 0)
		APPCONTROLLER->getImageCache()->request(new ImageCacheRequest(rssItem->imageurl, this));
	else
		mImage->setResource(BLANK);


	mTitle->setCaption(rssItem->title);
	mText->setCaption(rssItem->description);

	if(rssItem->url.length() > 0)
	{
		mLink->setCaption("Press * to read more");
		mUrl = rssItem->url;
	}
	else
	{
		mLink->setCaption("");
		mUrl.clear();
	}

	//Update the size of the main content box;
	mTitle->update();
	mText->update();
	mContent->setHeight(APPCONTROLLER->getScreenHeight() - mTitle->getHeight() - mSoftkey->getHeight());
}

void RSSScreen::keyPressEvent(int keyCode, int nativeCode)
{
	switch(keyCode)
	{
		case MAK_UP:
		case MAK_LEFT:
		case MAK_2:
		case MAK_4:
			mContent->selectPreviousItem();
			break;
		case MAK_DOWN:
		case MAK_RIGHT:
		case MAK_8:
		case MAK_6:
			mContent->selectNextItem();
			break;
		case MAK_SOFTRIGHT:
			APPCONTROLLER->back();
			break;
		case MAK_STAR:
			if(mUrl.length() > 0)
				maPlatformRequest(mUrl.c_str());
			break;
	}
}

