/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "RSSDownloadQueue.h"
#include "../std.h"

RSSDownloadQueue::RSSDownloadQueue()
				: mRequests(20)
{
	mProcessor = new RSSProcessor();
	//Listen to the processor so we know when it has finished
	mProcessor->setRSSProcessorListener(this);

	mConn = new XmlConnection();
	mConn->setXmlListener(mProcessor);
};

RSSDownloadQueue::~RSSDownloadQueue()
{
	mConn->close();
	delete mConn;
	delete mProcessor;
}

void RSSDownloadQueue::processQueue()
{
	//Don't process if one is already running
	if(mConn->isBusy())
		return;

	//Don't do anything if there are no requests
	if(mRequests.getCount() == 0)
		return;

	mNextRequest = mRequests.dequeue();
	mConn->setURL(mNextRequest->url.c_str());

	//Set the processor to send RSS items and channels to the requester.
	mProcessor->setRSSListener(mNextRequest->requester);

	//Process this XML
	mConn->beginDownloading();
}

void RSSDownloadQueue::processingComplete()
{
	//That Xml processing has finished, you can now try again
	delete mNextRequest;
	processQueue();
}

void RSSDownloadQueue::addRequest(String& url, RSSListener* requester)
{
	RSSDownloadRequest* newReq = new RSSDownloadRequest();
	newReq->url = url;
	newReq->requester = requester;

	mRequests.enqueue(newReq);

	processQueue();
}

void RSSDownloadQueue::addRequest(RSSDownloadRequest* req)
{
	mRequests.enqueue(req);

	processQueue();
}


