/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "RSSProcessor.h"
#include "../std.h"
#include <mastring.h>
#include <mastdlib.h>
#include <matime.h>
#include "masscanf.h"
#include "trim.h"
#include "DateTime.h"

RSSProcessor::RSSProcessor()
{}

RSSProcessor::~RSSProcessor()
{}

//XmlListener
void RSSProcessor::mtxEncoding(const char* value)
{}

void RSSProcessor::mtxTagStart(const char* name, int len)
{
  //LOG("Tag start: %s", name);
  if(mFirstTag.length() == 0)
  {
	  //This is the first tag.  When the matching end tag is called, we've finished
	  mFirstTag.append(name, len);
  }

  if(strncmp(name, "channel", 7) == 0
     || strncmp(name, "feed", 4) == 0)
  {
	//LOG("Setting to RSS_CHANNEL");
    mRssSection = RSS_CHANNEL;
    currentMax = 0;
    mChannelSent = false;
  }
  else if(strncmp(name, "title", 5) == 0)
  {
    if(mRssSection == RSS_CHANNEL)
    {
      //LOG("Setting to RSS_CHANNEL_TITLE");
      mRssTag = RSS_CHANNEL_TITLE;
    }
    else
    {
      //LOG("Setting to RSS_ITEM_TITLE");
      mRssTag = RSS_ITEM_TITLE;
    }
  }
  else if(strncmp(name, "desc", 4) == 0
          || strncmp(name, "summary", 7) == 0)
  {
    if(mRssSection == RSS_CHANNEL)
    {
      //LOG("Setting to RSS_CHANNEL_DESC");
      mRssTag = RSS_CHANNEL_DESC;
    }
	else
	{
	  //LOG("Setting to RSS_ITEM_DESC");
	  mRssTag = RSS_ITEM_DESC;
	}
  }
  else if(strncmp(name, "image", 5) == 0
      && mRssSection == RSS_CHANNEL)
  {
	  //LOG("Setting to RSS_CHANNEL_IMAGE");
	  mRssTag = RSS_CHANNEL_IMAGE;
  }
  else if(strncmp(name, "//LOGo", 4) == 0
      && mRssSection == RSS_CHANNEL)
  {
	  //LOG("Setting to RSS_CHANNEL_IMAGE_URL");
      mRssTag = RSS_CHANNEL_IMAGE_URL;
  }
  else if(strncmp(name, "url", 3) == 0
          || strncmp(name, "link", 4) == 0)
  {
	  if(mRssTag == RSS_CHANNEL_IMAGE)
	  {
		  //LOG("Setting to RSS_CHANNEL_IMAGE_URL");
          mRssTag = RSS_CHANNEL_IMAGE_URL;
	  }
	  else if(mRssTag == RSS_ITEM_IMAGE)
	  {
		  //LOG("Setting to RSS_ITEM_IMAGE_URL");
		  mRssTag = RSS_ITEM_IMAGE_URL;
	  }
	  else if(mRssSection == RSS_ITEM)
	  {
		  //LOG("Setting to RSS_ITEM_URL");
		  mRssTag = RSS_ITEM_URL;
	  }
  }
  else if(strncmp(name, "media:thumbnail", 15) == 0)
  {
	  //LOG("Found the image");
	  mRssTag = RSS_ITEM_IMAGE;
  }
  else if(strncmp(name, "item", 4) == 0
          || strncmp(name, "entry", 5) == 0)
  {
	//LOG("Setting to RSS_ITEM");
    mRssSection = RSS_ITEM;
    //new item, reset local data
    mCurrentItem.description.clear();
    mCurrentItem.title.clear();
    mCurrentItem.imageurl.clear();
    mCurrentItem.url.clear();
    if(!mChannelSent)
    {
    	//LOG("Sending channel");
    	//Send channel item
    	if(mListener != NULL)
    		mListener->rssChannelReceived(mCurrentChannel);
    	mChannelSent = true;
    }
  }
  else if(strncmp(name, "pubDate", 7) == 0
          || (strncmp(name, "updated", 7) == 0
						   && mRssSection == RSS_ITEM))
  {
	  mRssTag = RSS_ITEM_DATE;
	  //LOG("Found a pubDate");
  }
  else if(strncmp(name, "author", 6) == 0)
    mRssTag = RSS_CHANNEL_AUTHOR;
  else if(strncmp(name, "copyright", 9) == 0)
    mRssTag = RSS_CHANNEL_COPYRIGHT;

}

void RSSProcessor::mtxTagAttr(const char* attrName, const char* attrValue)
{
	//Item's image can come through as attributes
	if(strncmp(attrName, "url", 3) == 0
		&& mRssTag == RSS_ITEM_IMAGE)
	{
		mCurrentItem.imageurl = attrValue;
	}
}

void RSSProcessor::mtxTagStartEnd()
{
	if(mFirstTag.length() == 0)
	{
		//Empty XML document, report as complete
		mProcessorListener->processingComplete();
	}
}

void RSSProcessor::mtxTagData(const char* data, int len)
{
  //Add the data to the buffer, but don't process until we've got the end tag
  temp.append(data, len);
}

void RSSProcessor::mtxTagEnd(const char* name, int len)
{
  //LOG("Processing tag '%s'", name);
	//Clean up the data.  Apparently, RSS and HTML are the same thing...
  trim(temp);
  parseControlCodes(temp);
  stripHTML(temp);
  switch(mRssTag)
  {
    case RSS_CHANNEL_TITLE:
      //Rename the datasource to the title
      //LOG("Got the channel name %s", temp.c_str());
     // mCurrentItem.source = temp;
      mCurrentChannel.title = temp;
      break;
    case RSS_CHANNEL_DESC:
      mCurrentChannel.description = temp;
      break;
    case RSS_CHANNEL_IMAGE_URL:
      mCurrentChannel.imageurl = temp;
      break;
    case RSS_CHANNEL_AUTHOR:
      mCurrentChannel.author = temp;
      break;
    case RSS_CHANNEL_COPYRIGHT:
      mCurrentChannel.copyright = temp;
      break;
    case RSS_ITEM_TITLE:
      mCurrentItem.title = temp;
      break;
    case RSS_ITEM_DESC:
      mCurrentItem.description = temp;
      break;
    case RSS_ITEM_DATE:
      mCurrentItem.date = convertToDate(temp);
      //LOG("I think the date is '%s'", sprint_time(mCurrentItem.date));
      currentMax = currentMax < mCurrentItem.date ? mCurrentItem.date : currentMax;
      break;
    case RSS_ITEM_IMAGE_URL:
      mCurrentItem.imageurl = temp;
      break;
    case RSS_ITEM_URL:
      mCurrentItem.url = temp;
  }

  mRssTag = RSS_NULL;

  //Reset the buffer
  temp.clear();

  //LOG("Tag end: %s", name);

  if(strncmp(name, "item", 4) == 0
     || strncmp(name, "entry", 5) == 0)
    if(mCurrentItem.date > mLastUpdated)
    {
    	//LOG("Sending item");
      if(mListener != NULL)
    	  mListener->rssItemReceived(mCurrentItem);
    }

  if(strncmp(name, "feed", 4) == 0)
    mLastUpdated = currentMax;

  if(mFirstTag == name)
  {
	  //This tag matches the first one.  We've finished this document
	  mFirstTag.clear();
	  mProcessorListener->processingComplete();
  }
}

void RSSProcessor::mtxParseError()
{
  //LOG("Parse error");
}

void RSSProcessor::mtxEmptyTagEnd()
{
}

unsigned char RSSProcessor::mtxUnicodeCharacter(int unicode)
{
	XmlListener::mtxUnicodeCharacter(unicode);
}


void RSSProcessor::setRSSListener(RSSListener* listener)
{
	mListener = listener;
}

void RSSProcessor::setRSSProcessorListener(RSSProcessorListener* listener)
{
	mProcessorListener = listener;
}

time_t RSSProcessor::convertToDate(String& timestr)
{
	time_t returnTime;

    //LOG("Attempting date '%s'", temp.c_str());
    //OK - two different date formats
    // 1. Fri, 10 January 2010....
    // 2. 2010-02-24T13:59:05Z
    // Rubbish hack solution to tell the difference
    if(isdigit(temp.c_str()[0]))
    {
      //Type 2
  	  tm tim;
  	  sscanf(temp.c_str(), "%4d-%2d-%2dT%2d:%2d:%2dZ",
  			  &tim.tm_year,
  			  &tim.tm_mon,
  			  &tim.tm_mday,
  			  &tim.tm_hour,
  			  &tim.tm_min,
  			  &tim.tm_sec
  			  );

  	  returnTime = mktime(&tim);
    }
    else
    {
      //Type 1
      DateTime* dt = new DateTime(temp.c_str());
      //LOG("Passed it through DateTime");
      //LOG("Setting item date %s", sprint_time(dt->getTicks()));
      returnTime = dt->getTicks();
      delete dt;
    }

    return returnTime;
}
