/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#ifndef RSSPROCESSOR_H_
#define RSSPROCESSOR_H_
#include <MTXml/MTXml.h>
#include <MAUtil/String.h>
#include "RSS.h"

using namespace MAUtil;
using namespace Mtx;

class RSSProcessorListener
{
	public:
		virtual void processingComplete() = 0;
};

class RSSProcessor : public XmlListener
{
  public:
    RSSProcessor();
    ~RSSProcessor();

    //XmlListener
    void mtxEncoding(const char* value);
    void mtxTagStart(const char* name, int len);
    void mtxTagAttr(const char* attrName, const char* attrValue);
    void mtxTagStartEnd();
    void mtxTagData(const char* data, int len);
    void mtxTagEnd(const char* name, int len);
    void mtxParseError();
    void mtxEmptyTagEnd();
    unsigned char mtxUnicodeCharacter(int unicode);

    //Property setters
    void setRSSListener(RSSListener* listener);
    void setRSSProcessorListener(RSSProcessorListener* listener);

  private:
    enum RSS_Section
    {
    	RSS_CHANNEL,
    	RSS_ITEM
    } mRssSection;

    enum RSS_Tags
    {
    	RSS_NULL,
    	RSS_CHANNEL_TITLE,
    	RSS_CHANNEL_DESC,
    	RSS_CHANNEL_IMAGE,
    	RSS_CHANNEL_IMAGE_URL,
    	RSS_CHANNEL_AUTHOR,
    	RSS_CHANNEL_COPYRIGHT,
    	RSS_ITEM_TITLE,
    	RSS_ITEM_DESC,
    	RSS_ITEM_DATE,
    	RSS_ITEM_IMAGE,
    	RSS_ITEM_IMAGE_URL,
    	RSS_ITEM_URL
    } mRssTag;

    RSSListener* mListener;
    RSSProcessorListener* mProcessorListener;
    RSSItem mCurrentItem;
    RSSChannel mCurrentChannel;
    String temp;
    bool mChannelSent;
    String mSourceUrl;
    String mFirstTag;

    time_t mLastUpdated;
    time_t currentMax;

    time_t convertToDate(String& timestr);
};


#endif /* PODCASTPROCESSOR_H_ */
