/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "trim.h"
#include <mastring.h>
#include "Convert.h"
#include <MAUtil/Util.h>

void trim(String& instr)
{
  const char* p = instr.c_str();
  if(strlen(p) == 0)
    return;
  int start = 0;
  int len = strlen(p);
  int end = strlen(p);
  while(p[start] < 33 && start < len) start++;
  while(p[end] < 33 && end > start) end--;
  if(end == start)
    instr.clear();
  else
    instr = instr.substr(start, end - start + 1);
}

char* strReplaceFirst(char *str, const char* what, const char* to)
{
    int i, j, k, m, n, delta;
    int n1, n2, n3;
    char* p, *q;
    if (!str || !what)
        return 0;
    if (!to)
        to = "";
    n1 = strlen(str);
    n2 = strlen(what);
    n = n1 - n2 + 1;
    for (i = 0; i < n; ++i)
    {
        for (j = 0; j < n2 && what[j] == str[i+j]; ++j)
            ;
        if (j == n2) // found
        {
            n3 = strlen(to);
            delta = n3 - n2;
            m = n1 - i - n2;
            if (delta < 0) /* move left */
            {
                p = str + (i + n2 + delta);
                q = p - delta;
                for (k = 0; k <= m; ++k)
                    p[k] = q[k];
            }
            else if (delta > 0) /* move right */
            {
                q = str + n1 - m;
                p = q + delta;
                for (k = m; k >= 0; --k)
                    p[k] = q[k];
            }
            for (k = 0; k < n3; ++k)
                str[i+k] = to[k];
            return str + i + n3;
        }
    }
    return 0;
}

char* strReplace(char* str, const char* what, const char* to)
{
    char* p = str;
    while ((p=strReplaceFirst(p,what,to)) != 0)
        ;
    return str;
}

void parseControlCodes(String inp)
{
  char* p = (char*)inp.c_str();

  ////lprintfln("Started as: '%s'", p);
  p = strReplace(p, "&amp;", "&");
  p = strReplace(p, "&quot;", "\"");
  p = strReplace(p, "&lt;", "<");
  p = strReplace(p, "&gt;", ">");

  //Acute accents
  p = strReplace(p, "&Aacute;", "");
  p = strReplace(p, "&aacute;", "");
  p = strReplace(p, "&Eacute;", "");
  p = strReplace(p, "&eacute;", "");
  p = strReplace(p, "&Iacute;", "");
  p = strReplace(p, "&iacute;", "");
  p = strReplace(p, "&Oacute;", "");
  p = strReplace(p, "&oacute;", "");
  p = strReplace(p, "&Uacute;", "");
  p = strReplace(p, "&uacute;", "");
  p = strReplace(p, "&Yacute;", "");
  p = strReplace(p, "&yacute;", "");

  //Grave accents
  p = strReplace(p, "&Agrave;", "");
  p = strReplace(p, "&agrave;", "");
  p = strReplace(p, "&Egrave;", "");
  p = strReplace(p, "&egrave;", "");
  p = strReplace(p, "&Igrave;", "");
  p = strReplace(p, "&igrave;", "");
  p = strReplace(p, "&Ograve;", "");
  p = strReplace(p, "&ograve;", "");
  p = strReplace(p, "&Ugrave;", "");
  p = strReplace(p, "&ugrave;", "");

  //Circumflex accents
  p = strReplace(p, "&Acirc;", "");
  p = strReplace(p, "&acirc;", "");
  p = strReplace(p, "&Ecirc;", "");
  p = strReplace(p, "&ecirc;", "");
  p = strReplace(p, "&Icirc;", "");
  p = strReplace(p, "&icirc;", "");
  p = strReplace(p, "&Ocirc;", "");
  p = strReplace(p, "&ocirc;", "");
  p = strReplace(p, "&Ucirc;", "");
  p = strReplace(p, "&ucirc;", "");

  //Umlaut accents
  p = strReplace(p, "&Auml;", "");
  p = strReplace(p, "&auml;", "");
  p = strReplace(p, "&Euml;", "");
  p = strReplace(p, "&euml;", "");
  p = strReplace(p, "&Iuml;", "");
  p = strReplace(p, "&iuml;", "");
  p = strReplace(p, "&Ouml;", "");
  p = strReplace(p, "&ouml;", "");
  p = strReplace(p, "&Uuml;", "");
  p = strReplace(p, "&uuml;", "");
  //It appears that &Yuml; isn't a recognised character, but I've included it anyway
  p = strReplace(p, "&Yuml;", "");
  p = strReplace(p, "&yuml;", "");

  //Tilde accents
  p = strReplace(p, "&Ntilde;", "");
  p = strReplace(p, "&ntilde;", "");
  p = strReplace(p, "&Otilde;", "");
  p = strReplace(p, "&otilde;", "");

  //Cedilla accents
  p = strReplace(p, "&Ccedil;", "");
  p = strReplace(p, "&ccedil;", "");

  //Other letters
  p = strReplace(p, "&AElig;", "");
  p = strReplace(p, "&aelig;", "");
  p = strReplace(p, "&OElig;", "");
  p = strReplace(p, "&oelig;", "");
  p = strReplace(p, "&szlig;", "");
  p = strReplace(p, "&ETH;", "");
  p = strReplace(p, "&eth;", "");
  p = strReplace(p, "&THORN;", "");
  p = strReplace(p, "&thorn;", "");
  p = strReplace(p, "&Aring;", "");
  p = strReplace(p, "&aring;", "");
  p = strReplace(p, "&Oslash;", "");
  p = strReplace(p, "&oslash;", "");

  //Puncuation
  p = strReplace(p, "&euro;", "");
  p = strReplace(p, "&pound;", "");
  p = strReplace(p, "&laquo;", "");
  p = strReplace(p, "&raquo;", "");
  p = strReplace(p, "&bull;", "");
  p = strReplace(p, "&dagger;", "");
  p = strReplace(p, "&copy;", "");
  p = strReplace(p, "&reg;", "&reg");
  p = strReplace(p, "&deg;", "");
  p = strReplace(p, "&micro;", "");
  p = strReplace(p, "&middot;", "");
  p = strReplace(p, "&ndash;", "");
  p = strReplace(p, "&mdash;", "");
  p = strReplace(p, "&iexcl;", "");
  p = strReplace(p, "&iquest;", "");
  p = strReplace(p, "&nbsp;", " ");

  //lprintfln("Now: '%s'", p);

  int ccstart = inp.find("&#");
  int ccend;
  int ccode;
  char com[2];
  com[1] = 0;
  char* compointer = com;
  while(ccstart != inp.npos)
  {
    ccend = ccstart + 2;
    while(isdigit(inp.c_str()[ccend]))ccend++;
    //lprintfln("Found code: '%s'", inp.substr(ccstart, ccend - ccstart).c_str());
    ccode = stringToInteger(inp.substr(ccstart + 2, ccend - ccstart -2));
    if(ccode < 255)
      com[0] = (char)ccode;
    else
      com[0] = 0;

    //lprintfln("testing next char %c", inp.c_str()[ccend+1]);
    if(inp.c_str()[ccend] == ';')
      {
        //lprintfln("removing semicolon");
        ccend++;
      }

    strReplace(p, inp.substr(ccstart, ccend - ccstart).c_str(), compointer);
    //lprintfln("final: %s", p);

    inp = p;
    ccstart = inp.find("&#");
  }
}

void trimQuotes(String& val)
{
  trim(val);
  if(   val.c_str()[0] == '\"'
     && val.c_str()[val.length() -1] == '\"')
  {
    val.remove(val.length() -1, 1);
    val.remove(0, 1);
  }
  trim(val);
}

void stripHTML(String& inp)
{
  //lprintfln("Stripping %s", inp.c_str());
  parseControlCodes(inp);

  int openTag = 0;
  int closeTag = 0;

  while(inp.findFirstOf('>', 0) != -1)
  {
    openTag = inp.findFirstOf('<', 0);
    closeTag = inp.findFirstOf('>', openTag) + 1;
    //lprintfln("removing HTML '%s'", inp.substr(openTag, closeTag - openTag).c_str());
    inp = strReplace(inp.pointer(), inp.substr(openTag, closeTag - openTag).c_str(), "");
  }
}
