/* Copyright (C) 2010 MoSync AB

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License, version 2, as published by
the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA
02111-1307, USA.
*/

#include "RSSItemWidget.h"
#include "KineticListBox.h"
#include "../std.h"

RSSItemWidget::RSSItemWidget(int x, int y, int width, int height, Widget* parent)
	   : Widget(x, y, width, height, parent),
	     mNextImage(NULL),
	     mListening(false)
{
	mTitle = new Label(0, 0, width, height, this);
	mTitle->setDrawBackground(false);
	mTitle->setMultiLine(true);
	mTitle->setAutoSizeX(false);
	mTitle->setAutoSizeY(true);
	mTitle->setPaddingLeft(3);
	mTitle->setPaddingRight(3);
	mTitle->setPaddingBottom(3);
	mText = new Label(0, 0, width, height, this);
	mText->setDrawBackground(false);
	mText->setMultiLine(true);
	mText->setAutoSizeX(false);
	mText->setAutoSizeY(true);
	mText->setPaddingLeft(3);
	mText->setPaddingRight(3);
	mText->setPaddingBottom(5);

}

RSSItemWidget::~RSSItemWidget()
{
	if(mListening)
		Environment::getEnvironment().removePointerListener(this);
}

void RSSItemWidget::setText(String& text)
{
	mText->setCaption(text);
	mText->update();
}

void RSSItemWidget::setTextFont(Font* font)
{
	mText->setFont(font);
	mText->update();
}

void RSSItemWidget::setTitle(String& title)
{
	mTitle->setCaption(title);
	mTitle->update();
	this->setHeight(mTitle->getHeight());
}

void RSSItemWidget::setTitleFont(Font* font)
{
	mTitle->setFont(font);
	mTitle->update();
	this->setHeight(mTitle->getHeight());
}

//Makes sure that the next image is in the correct place.  Ensure that you've sent the title
//and description text and fonts before you set this image, so that it can be placed correctly.
void RSSItemWidget::setNextImage(MAHandle nextImage)
{
	//
	if(mNextImage == NULL)
		mNextImage = new Image(0, 0, 20, 20, this, true, true, nextImage);
	else
		mNextImage->setResource(nextImage);

	//Updates the dimensions of the image
	mNextImage->update();

	//Set the size of the text boxes
	mText->setWidth(this->getWidth() - mNextImage->getWidth());
	mTitle->setWidth(mText->getWidth());

	//Updates the dimensions of the text boxes
	mText->update();
	mTitle->update();

	//Put the image to the right of the text boxes, and aligned with the title
	mNextImage->setPosition(this->getWidth() - mNextImage->getWidth(), 0);
}

void RSSItemWidget::setUrl(String& url)
{
	mUrl = url;
}

void RSSItemWidget::setImageUrl(String& url)
{
	mImageUrl = url;
}

const String& RSSItemWidget::getText()
{
	return mText->getCaption();
}

const String& RSSItemWidget::getTitle()
{
	return mTitle->getCaption();
}

String& RSSItemWidget::getUrl()
{
	return mUrl;
}

String& RSSItemWidget::getImageUrl()
{
	return mImageUrl;
}


void RSSItemWidget::setSelected(bool selected)
{
	if(selected)
	{
		this->setHeight(mTitle->getHeight() + mText->getHeight());
		mText->setPosition(0, mTitle->getHeight());
	}
	else
		this->setHeight(mTitle->getHeight());

	Widget::setSelected(selected);

	if(selected)
	{
		if(!mListening)
		{
			Environment::getEnvironment().addPointerListener(this);
			mListening = true;
		}
	}
	else
	{
		if(mListening)
		{
			Environment::getEnvironment().removePointerListener(this);
			mListening = false;
		}
	}
	this->getParent()->update();
}

void RSSItemWidget::pointerPressEvent(MAPoint2d p)
{}

void RSSItemWidget::pointerMoveEvent(MAPoint2d p)
{}

void RSSItemWidget::pointerReleaseEvent(MAPoint2d p)
{
	mNextImage->update();
	KineticListBox* klb = (KineticListBox*)this->getParent();
	Point testpos = mNextImage->getPosition();

	//If you reuse this component, and your widget hierarchy is different, then you
	//will need to recalculate this.
	int adjustedY = testpos.y + this->getPosition().y + this->getParent()->getPosition().y + (klb->getYOffset() >> 16);

	//Inform WidgetListeners if the next item is pressed
	if(mNextImage != NULL)
		if(p.x >= testpos.x && p.x <= testpos.x + mNextImage->getWidth()
		&& p.y >= adjustedY && p.y <= adjustedY + mNextImage->getHeight())
			trigger();
}

//Make the appropriate widgets visible when selected.
void RSSItemWidget::drawWidget()
{
	mText->setEnabled(this->isSelected());
	mNextImage->setEnabled(this->isSelected());
	update();
}

